@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2007 Zeeshan Ali <zeenix@gstreamer.net>
+ * Copyright (C) 2007 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *
- * Authors: Zeeshan Ali <zeenix@gstreamer.net>
+ * Authors: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,6 +49,8 @@ static GUPnPDIDLLiteParser *didl_parser;
 
 typedef struct
 {
+  GUPnPServiceProxy *content_dir;
+
   gchar *id;
 
   guint32 starting_index;
@@ -64,12 +66,14 @@ typedef struct
 } BrowseMetadataData;
 
 static BrowseData *
-browse_data_new (const char *id,
-                 guint32     starting_index)
+browse_data_new (GUPnPServiceProxy *content_dir,
+                 const char        *id,
+                 guint32            starting_index)
 {
         BrowseData *data;
 
         data = g_slice_new (BrowseData);
+        data->content_dir = g_object_ref (content_dir);
         data->id = g_strdup (id);
         data->starting_index = starting_index;
 
@@ -80,6 +84,7 @@ static void
 browse_data_free (BrowseData *data)
 {
         g_free (data->id);
+        g_object_unref (data->content_dir);
         g_slice_free (BrowseData, data);
 }
 
@@ -296,12 +301,13 @@ tree_selection_func (GtkTreeSelection *selection,
 }
 
 void
-setup_playlist_treeview (GladeXML *glade_xml)
+setup_playlist_treeview (GtkBuilder *builder)
 {
         GtkTreeModel      *model;
         GtkTreeSelection  *selection;
 
-        treeview = glade_xml_get_widget (glade_xml, "playlist-treeview");
+        treeview = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                       "playlist-treeview"));
         g_assert (treeview != NULL);
 
         model = create_playlist_treemodel ();
@@ -545,10 +551,10 @@ on_container_update_ids (GUPnPServiceProxy *content_dir,
 }
 
 static void
-append_didle_object (xmlNode           *object_node,
-                     GUPnPServiceProxy *content_dir,
-                     GtkTreeModel      *model,
-                     GtkTreeIter       *server_iter)
+append_didl_object (xmlNode      *object_node,
+                    BrowseData   *browse_data,
+                    GtkTreeModel *model,
+                    GtkTreeIter  *server_iter)
 {
         GtkTreeIter parent_iter;
         char       *id;
@@ -586,26 +592,28 @@ append_didle_object (xmlNode           *object_node,
                 icon = get_item_icon (object_node);
         }
 
-        if (!find_row (model,
-                       server_iter,
-                       &parent_iter,
-                       compare_container,
-                       (gpointer) parent_id,
-                       TRUE)) {
-                /* Assume parent to be the root container if parentID is
-                 * unknown */
+        /* Check if we browsed the root container. */
+        if (strcmp (browse_data->id, "0") == 0) {
                 parent_iter = *server_iter;
+        } else if (!find_row (model,
+                              server_iter,
+                              &parent_iter,
+                              compare_container,
+                              (gpointer) parent_id,
+                              TRUE)) {
+                goto return_point;
         }
 
         gtk_tree_store_insert_with_values (GTK_TREE_STORE (model),
                                            NULL, &parent_iter, position,
                                            0, icon,
                                            1, title,
-                                           3, content_dir,
+                                           3, browse_data->content_dir,
                                            4, id,
                                            5, is_container,
                                            -1);
 
+return_point:
         g_free (parent_id);
         g_free (title);
         g_free (id);
@@ -657,17 +665,17 @@ on_didl_object_available (GUPnPDIDLLiteParser *didl_parser,
                           xmlNode             *object_node,
                           gpointer             user_data)
 {
-        GUPnPServiceInfo *content_dir;
-        GtkTreeModel     *model;
-        GtkTreeIter       server_iter;
-        const char       *udn;
+        BrowseData   *browse_data;
+        GtkTreeModel *model;
+        GtkTreeIter   server_iter;
+        const char   *udn;
 
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
         g_assert (model != NULL);
 
-        content_dir = GUPNP_SERVICE_INFO (user_data);
+        browse_data = (BrowseData *) user_data;
 
-        udn = gupnp_service_info_get_udn (content_dir);
+        udn = gupnp_service_info_get_udn (GUPNP_SERVICE_INFO (browse_data->content_dir));
 
         if (find_row (model,
                        NULL,
@@ -675,10 +683,10 @@ on_didl_object_available (GUPnPDIDLLiteParser *didl_parser,
                        compare_media_server,
                        (gpointer) udn,
                        FALSE)) {
-                append_didle_object (object_node,
-                                     GUPNP_SERVICE_PROXY (content_dir),
-                                     model,
-                                     &server_iter);
+                append_didl_object (object_node,
+                                    browse_data,
+                                    model,
+                                    &server_iter);
         }
 
         return;
@@ -722,7 +730,7 @@ browse_cb (GUPnPServiceProxy       *content_dir,
                                                 (didl_parser,
                                                  didl_xml,
                                                  on_didl_object_available,
-                                                 content_dir,
+                                                 data,
                                                  &error)) {
                         g_warning ("%s\n", error->message);
                         g_error_free (error);
@@ -752,7 +760,6 @@ browse_cb (GUPnPServiceProxy       *content_dir,
         }
 
         browse_data_free (data);
-        g_object_unref (content_dir);
 }
 
 static void
@@ -763,9 +770,9 @@ browse (GUPnPServiceProxy *content_dir,
 {
         BrowseData *data;
 
-        g_object_ref (content_dir);
-
-        data = browse_data_new (container_id, starting_index);
+        data = browse_data_new (content_dir,
+                                container_id,
+                                starting_index);
 
         gupnp_service_proxy_begin_action
                                 (content_dir,
