@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2007 Zeeshan Ali <zeenix@gstreamer.net>
+ * Copyright (C) 2007 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *
- * Authors: Zeeshan Ali <zeenix@gstreamer.net>
+ * Authors: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -262,6 +262,26 @@ set_volume_hscale (guint volume)
                                            NULL);
 }
 
+static gboolean
+mime_type_is_a (const char *mime_type1, const char *mime_type2)
+{
+        gboolean ret;
+
+        char *content_type1 = g_content_type_from_mime_type (mime_type1);
+        char *content_type2 = g_content_type_from_mime_type (mime_type2);
+        if (content_type1 == NULL || content_type2 == NULL) {
+                /* Uknown content type, just do a simple comarison */
+                ret = g_ascii_strcasecmp (mime_type1, mime_type2) == 0;
+        } else {
+                ret = g_content_type_is_a (content_type1, content_type2);
+        }
+
+        g_free (content_type1);
+        g_free (content_type2);
+
+        return ret;
+}
+
 static gboolean
 is_transport_compat (const gchar *renderer_protocol,
                      const gchar *renderer_host,
@@ -285,8 +305,7 @@ is_content_format_compat (const gchar *renderer_content_format,
                           const gchar *item_content_format)
 {
         if (g_ascii_strcasecmp (renderer_content_format, "*") != 0 &&
-            g_ascii_strcasecmp (renderer_content_format,
-                                item_content_format) != 0) {
+            !mime_type_is_a (item_content_format, renderer_content_format)) {
                 return FALSE;
         } else {
                 return TRUE;
@@ -356,19 +375,19 @@ no_item_tokens:
 }
 
 
-static gint
-protocol_equal_func (xmlNode     *res_node,
-                     const gchar *renderer_protocol)
+static gboolean
+is_protocol_info_compat (xmlNode     *res_node,
+                         const gchar *renderer_protocol)
 {
         gchar *item_protocol;
         gchar **item_proto_tokens;
         gchar **renderer_proto_tokens;
-        gint ret = -1;
+        gboolean ret = FALSE;
 
         item_protocol = gupnp_didl_lite_property_get_attribute (res_node,
                                                                 "protocolInfo");
         if (item_protocol == NULL) {
-                return -1;
+                return FALSE;
         }
 
         item_proto_tokens = g_strsplit (item_protocol,
@@ -396,7 +415,7 @@ protocol_equal_func (xmlNode     *res_node,
                                       item_proto_tokens[2]) &&
             is_additional_info_compat (renderer_proto_tokens[3],
                                        item_proto_tokens[3])) {
-                ret = 0;
+                ret = TRUE;
         }
 
 return_point:
@@ -407,6 +426,40 @@ return_point:
         return ret;
 }
 
+static gboolean
+is_resource_compat (xmlNode *res_node, char **protocols)
+{
+        gboolean ret = FALSE;
+        int i;
+
+        for (i = 0; protocols[i]; i++) {
+                if (is_protocol_info_compat (res_node, protocols[i])) {
+                        ret = TRUE;
+                        break;
+                }
+        }
+
+        return ret;
+}
+
+static GList *
+find_compatible_res (GList *resources, char **protocols)
+{
+        GList *res;
+        GList *ret = NULL;
+
+        for (res = resources; res != NULL; res = res->next) {
+                xmlNode *res_node = (xmlNode *) res->data;
+
+                if (is_resource_compat (res_node, protocols)) {
+                        ret = res;
+                        break;
+                }
+        }
+
+        return ret;
+}
+
 static char *
 find_compat_uri_from_res (GList *resources, char **duration)
 {
@@ -426,9 +479,7 @@ find_compat_uri_from_res (GList *resources, char **duration)
                 GList   *res;
                 xmlNode *res_node;
 
-                res = g_list_find_custom (resources,
-                                          protocols[i],
-                                          (GCompareFunc) protocol_equal_func);
+                res = find_compatible_res (resources, protocols);
                 if (res == NULL) {
                         continue;
                 }
@@ -915,27 +966,35 @@ on_volume_vscale_value_changed (GtkRange *range,
 }
 
 void
-setup_renderer_controls (GladeXML *glade_xml)
+setup_renderer_controls (GtkBuilder *builder)
 {
-        volume_vscale = glade_xml_get_widget (glade_xml, "volume-vscale");
+        volume_vscale = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                            "volume-vscale"));
         g_assert (volume_vscale != NULL);
 
-        position_hscale = glade_xml_get_widget (glade_xml, "position-hscale");
+        position_hscale = GTK_WIDGET (gtk_builder_get_object (
+                                                builder,
+                                                "position-hscale"));
         g_assert (position_hscale != NULL);
 
-        play_button = glade_xml_get_widget (glade_xml, "play-button");
+        play_button = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                          "play-button"));
         g_assert (play_button != NULL);
 
-        pause_button = glade_xml_get_widget (glade_xml, "pause-button");
+        pause_button = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                           "pause-button"));
         g_assert (pause_button != NULL);
 
-        stop_button = glade_xml_get_widget (glade_xml, "stop-button");
+        stop_button = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                          "stop-button"));
         g_assert (stop_button != NULL);
 
-        next_button = glade_xml_get_widget (glade_xml, "next-button");
+        next_button = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                          "next-button"));
         g_assert (next_button != NULL);
 
-        prev_button = glade_xml_get_widget (glade_xml, "previous-button");
+        prev_button = GTK_WIDGET (gtk_builder_get_object (builder,
+                                                          "previous-button"));
         g_assert (prev_button != NULL);
 
         didl_parser = gupnp_didl_lite_parser_new ();
