@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2007 Zeeshan Ali <zeenix@gstreamer.net>
+ * Copyright (C) 2007 Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *
- * Authors: Zeeshan Ali <zeenix@gstreamer.net>
+ * Authors: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,14 +43,14 @@ device_proxy_unavailable_cb (GUPnPControlPoint *cp,
 static gboolean
 init_upnp (void)
 {
-        GError *error;
+        GError *error = NULL;
 
         g_type_init ();
 
-        error = NULL;
         context = gupnp_context_new (NULL, NULL, 0, &error);
         if (error) {
-                g_critical (error->message);
+                g_printerr ("Error creating the GUPnP context: %s\n",
+			    error->message);
                 g_error_free (error);
 
                 return FALSE;
